#include <Wire.h>
#include <SoftwareSerial.h>
#include <RedMP3.h>
#include "RTClib.h"
#include <avr/sleep.h>

// uncomment this line on the first upload or if clock changes due to daylight savings
// (and you're the kind of person to undertake such a sisyphean task)
// after uploading with the line uncommented, be sure to upload again with the line commmented out,
// or else on every Arduino reset, the clock will reset itself to the time of the last upload...
// #define ADJUST_DATE_TIME_NOW

// Vocal time encoding scheme.
// the following 4 arrays (ROUND_HOUR, QUARTER_PAST, HALF_PAST, QUARTER_TO)
// correspond to the 15 minutes resolution this clock has
//
// The numbers inside the array are as follows:
// CURRENT_HOUR is a place holder for current hour.
// SAY_NOTHING means nothing to play
// all others numbers are track numbers to be played
// currently my convention is: 20: "it is now", 21: "quarter past", 22: "half past", 23: "quarter to", 24: "o'clock"

#define MAX_TRACK_COUNT 3
#define SAY_NOTHING 0
#define CURRENT_HOUR 99

// HEBREW
/*
  #define MP3_FOLDER 1
  byte ROUND_HOUR[MAX_TRACK_COUNT] = {20, CURRENT_HOUR, SAY_NOTHING};
  byte QUARTER_PAST[MAX_TRACK_COUNT] = {20, CURRENT_HOUR, 21};
  byte HALF_PAST[MAX_TRACK_COUNT] = {20, CURRENT_HOUR, 22};
  byte QUARTER_TO[MAX_TRACK_COUNT] = {20, CURRENT_HOUR, 99};
*/

// ENGLISH
#define MP3_FOLDER 2
byte ROUND_HOUR[MAX_TRACK_COUNT] = {20, CURRENT_HOUR, 24};
byte QUARTER_PAST[MAX_TRACK_COUNT] = {20, 21, CURRENT_HOUR};
byte HALF_PAST[MAX_TRACK_COUNT] = {20, 22, CURRENT_HOUR};
byte QUARTER_TO[MAX_TRACK_COUNT] = {20, 23, CURRENT_HOUR};

#define BUTTON_PIN 2
#define AUDIO_POWER_PIN 4

#define VOLUME 0x18
#define TRACK_TIME_IS 20
#define TRACK_AND_QUARTER 21
#define TRACK_AND_HALF 22
#define TRACK_QUARTER_TO 23
#define VOLUME_PIN A0

#define MP3_RX 5
#define MP3_TX 6

MP3 mp3(MP3_RX, MP3_TX);

RTC_DS3231 RTC;

void setup () {
  Serial.begin(9600);
  pinMode(BUTTON_PIN, INPUT_PULLUP);
  pinMode(AUDIO_POWER_PIN, OUTPUT);
  Wire.begin();
  RTC.begin();
#ifdef ADJUST_DATE_TIME_NOW
  Serial.print("Adjusting to: ");
  Serial.print(F(__DATE__));
  Serial.print(" ");
  Serial.print(F(__TIME__));
  Serial.println();
  RTC.adjust(DateTime(F(__DATE__), F(__TIME__)));
#endif
  delay(500);
  sleepNow();
}

void loop() {
  digitalWrite(AUDIO_POWER_PIN, HIGH);
  delay(1000);
  // mp3.setVolume(analogRead(VOLUME_PIN) >> 5);
  mp3.setVolume(VOLUME);
  sayTime();
  while (digitalRead(BUTTON_PIN)) delay(100);
  digitalWrite(AUDIO_POWER_PIN, LOW);

  // not a typo. actually needed.
  // yes, it is probably a bug lurking somewhere.
  // but it works now, so TODO that later
  sleepNow();
  sleepNow();
}

void sayTime() {
  DateTime now = RTC.now();
  int h = now.hour();
  if (h > 12) h -= 12;
  if (h == 0) h = 12;
  int m = now.minute();
  Serial.println(String(h) + ':' + m);
  int quarterResolution = -1;
  int targetHour = -1;

  // play(MP3_FOLDER, TRACK_TIME_IS);
  if (m >= 50 || m < 10) {
    quarterResolution = 0;
    targetHour = (m >= 50) ? (h + 1) : h;
    // play(MP3_FOLDER, (m >= 50) ? (h + 1) : h);
  }
  else if (m >= 10 && m < 20) {
    quarterResolution = 1;
    targetHour = h;
    //    play(MP3_FOLDER, h);
    //    play(MP3_FOLDER, TRACK_AND_QUARTER);
  }
  else if (m >= 20 && m < 40) {
    quarterResolution = 2;
    targetHour = h;
    //    play(MP3_FOLDER, h);
    //    play(MP3_FOLDER, TRACK_AND_HALF);
  }
  else if (m >= 40 && m < 50) {
    quarterResolution = 3;
    targetHour = h + 1;
    //    play(MP3_FOLDER, TRACK_QUARTER_TO);
    //    play(MP3_FOLDER, h + 1);
  }
  switch (quarterResolution) {
    case 0:
      announce(ROUND_HOUR, targetHour);
      break;
    case 1:
      announce(QUARTER_PAST, targetHour);
      break;
    case 2:
      announce(HALF_PAST, targetHour);
      break;
    case 3:
      announce(QUARTER_TO, targetHour);
      break;
  }
}

void announce (byte grammer[], int targetHour) {
  // Serial.println(String("targetHour: ") + targetHour);
  for (int i = 0; i < MAX_TRACK_COUNT; ++i) {
    // Serial.println(grammer[i]);
    if (grammer[i] == SAY_NOTHING) {
      continue;
    }
    else if (grammer[i] == CURRENT_HOUR) {
      play(MP3_FOLDER, targetHour);
    }
    else {
      play(MP3_FOLDER, grammer[i]);
    }
  }
}

void play(int folder, int track) {
  mp3.playWithFileName(folder, track);
  delay(50);
  while (mp3.getStatus() != STATUS_STOP) {
    delay(50);
  }
}

void wake () {
  sleep_disable();
  detachInterrupt (digitalPinToInterrupt(BUTTON_PIN));
}

void sleepNow() {
  set_sleep_mode (SLEEP_MODE_PWR_DOWN);
  noInterrupts();
  sleep_enable();
  attachInterrupt(digitalPinToInterrupt(BUTTON_PIN), wake, HIGH);
  interrupts();
  sleep_cpu();
}

